import { HttpClient } from '@angular/common/http';
import { Injectable, ɵɵdefineInjectable, ɵɵinject, APP_INITIALIZER, isDevMode } from '@angular/core';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { QueryResult, EditResult } from '@yukawa/chain-base-angular-domain';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ConfigService {
    /**
     * @param {?} http
     */
    constructor(http) {
        this.http = http;
        this.configUrl = "config.json";
        //this.initConfig();
    }
    /**
     * @return {?}
     */
    initConfig() {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            this.loadConfig()
                .subscribe((/**
             * @param {?} data
             * @return {?}
             */
            (data) => {
                this.config = data;
                resolve(true);
            }), (/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                console.error(`ConfigService initConfig  ${error}`);
                return Observable.throw(error || 'Server error');
            }));
        }));
    }
    /**
     * @param {?=} id
     * @return {?}
     */
    getBaseUrl(id = "") {
        return id == "" ? this.config.baseUrl : this.config[id];
    }
    /**
     * @param {?=} id
     * @return {?}
     */
    getConfig(id) {
        if (!id)
            return this.config;
        return `${this.getBaseUrl()}${this.config[id]}`;
    }
    /**
     * @return {?}
     */
    loadConfig() {
        return this.http.get(this.configUrl);
    }
}
ConfigService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
ConfigService.ctorParameters = () => [
    { type: HttpClient }
];
/** @nocollapse */ ConfigService.ngInjectableDef = ɵɵdefineInjectable({ factory: function ConfigService_Factory() { return new ConfigService(ɵɵinject(HttpClient)); }, token: ConfigService, providedIn: "root" });
if (false) {
    /**
     * @type {?}
     * @protected
     */
    ConfigService.prototype.configUrl;
    /** @type {?} */
    ConfigService.prototype.config;
    /**
     * @type {?}
     * @protected
     */
    ConfigService.prototype.http;
}
/**
 * @param {?} config
 * @return {?}
 */
function ConfigFactory(config) {
    return (/**
     * @return {?}
     */
    () => config.initConfig());
}
/**
 * @return {?}
 */
function init() {
    return {
        provide: APP_INITIALIZER,
        useFactory: ConfigFactory,
        deps: [ConfigService],
        multi: true
    };
}
/** @type {?} */
const ConfigModule = {
    init: init
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const TAG = 'RestAspect';
/**
 * @abstract
 */
class RestAspect {
    /**
     * @protected
     * @param {?} http
     * @param {?} configService
     * @param {?=} serviceUrl
     */
    constructor(http, configService, serviceUrl) {
        this.http = http;
        this.configService = configService;
        this.serviceUrl = serviceUrl;
    }
    /**
     * @protected
     * @param {?=} path
     * @return {?}
     */
    formatBaseUrl(path = '') {
        return this.configService.getConfig('baseUrl') + path;
    }
    /**
     * @protected
     * @param {?=} path
     * @return {?}
     */
    formatServiceUrl(path = '') {
        return this.serviceUrl + path + '/';
    }
    // noinspection JSMethodCanBeStatic
    /**
     * @return {?}
     */
    get verbose() {
        return isDevMode();
    }
    /**
     * @protected
     * @template E
     * @param {?} url
     * @param {?} filter
     * @param {?=} method
     * @return {?}
     */
    search(url, filter, method = 'post') {
        /** @type {?} */
        let m = method || 'post';
        if (this.verbose)
            console.debug(TAG, "search:", m, url, filter);
        return this.http[m](url, filter).pipe(map((/**
         * @param {?} r
         * @return {?}
         */
        (r) => {
            /** @type {?} */
            let result = (/** @type {?} */ ({
                took: (/** @type {?} */ (r.took)),
                timedOut: (/** @type {?} */ (r.timed_out)),
                rows: (/** @type {?} */ ([]))
            }));
            /** @type {?} */
            let h = r.hits;
            if (h) {
                result.total = h.total;
                for (let idx = 0; idx < h.hits.length; idx++) {
                    /** @type {?} */
                    const hit = h.hits[idx];
                    /** @type {?} */
                    let source = (/** @type {?} */ (hit._source));
                    // console.log('DeviceService', hit);
                    result.rows.push(source);
                }
            }
            return result;
        })));
    }
    /**
     * @protected
     * @template E
     * @param {?} url
     * @param {?} filter
     * @param {?=} options
     * @return {?}
     */
    query(url, filter, options = {}) {
        //options = this.defaultOptions(options);
        if (this.verbose)
            console.debug(TAG, "query:", url, filter);
        return this.http.post(url, filter, options).pipe(map((/**
         * @param {?} data
         * @return {?}
         */
        (data) => QueryResult.fromData(data))));
    }
    /**
     * @protected
     * @template E
     * @param {?} url
     * @param {?=} value
     * @param {?=} options
     * @return {?}
     */
    edit(url, value = null, options = {}) {
        if (this.verbose)
            console.debug(TAG, "edit:", url, value);
        return (options.method === 'put'
            ? this.http.put(url, value, options)
            : this.http.post(url, value, options)).pipe(map((/**
         * @param {?} data
         * @return {?}
         */
        (data) => EditResult.fromData(data))));
    }
    /**
     * @protected
     * @template E
     * @param {?} url
     * @param {?=} value
     * @param {?=} options
     * @return {?}
     */
    load(url, value = null, options = {}) {
        if (this.verbose)
            console.debug(TAG, "load:", url);
        /** @type {?} */
        let method = options.method || 'get';
        /** @type {?} */
        let obs;
        if (value) {
            obs = this.http[method](url, value, options);
        }
        else {
            obs = this.http[method](url, options);
        }
        return obs;
    }
    /**
     * @protected
     * @param {?} url
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    download(url, filter = null, options = {}) {
        if (this.verbose)
            console.debug(TAG, "download:", url, filter);
        options = options || {};
        options.responseType = options.responseType || (/** @type {?} */ ('blob'));
        /** @type {?} */
        let method = options.method || 'post';
        return this.http[method](url, filter, options);
    }
    /**
     * @protected
     * @param {?} value
     * @param {?=} message
     * @return {?}
     */
    checkNotNull(value, message = "Not found") {
        if (value == null)
            throw new Error(message);
        return value;
    }
}
if (false) {
    /**
     * @type {?}
     * @protected
     */
    RestAspect.prototype.http;
    /**
     * @type {?}
     * @protected
     */
    RestAspect.prototype.configService;
    /**
     * @type {?}
     * @protected
     */
    RestAspect.prototype.serviceUrl;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ConfigFactory, ConfigModule, ConfigService, RestAspect, init };
//# sourceMappingURL=yukawa-chain-base-angular-client.js.map
